﻿
namespace ProjectDumpTruck.MovementStrategy;

/// <summary>
/// Класс-страгетия перемещения объекта
/// </summary>
public abstract class AbstractStrategy
{

    /// <summary>
    /// Перемещаемый объект
    /// </summary>
    private IMoveableObject? _moveableObject;
    
    /// <summary>
    /// Статус перемещения
    /// </summary>
    private StrategyStatus _state = StrategyStatus.NotInit;

    /// <summary>
    /// Ширина поля
    /// </summary>
    protected int FieldWidth {  get; private set; }

    /// <summary>
    /// Высота поля
    /// </summary>
    protected int FieldHeight { get; private set;}

    /// <summary>
    /// Статус перемещения
    /// </summary>
    /// <returns></returns>
    public StrategyStatus GetStatus() {  return _state; }

    /// <summary>
    /// Установка данных
    /// </summary>
    /// <param name="moveableObject">Перемещаемый объект</param>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    public void SetData(IMoveableObject moveableObject, int width, int height)
    {
        if (moveableObject == null)
        {
            _state = StrategyStatus.NotInit;
            return;
        }

        _state = StrategyStatus.InProgress;
        _moveableObject = moveableObject;
        FieldWidth = width;
        FieldHeight = height;

    }

    /// <summary>
    /// Перемещение влево
    /// </summary>
    /// <returns>Результаты перемещения (true - удалось переместиться,false - неудача)</returns>
    protected bool MoveLeft() => MoveTo(MovementDirection.Left);

    /// <summary>
    /// Перемещение право
    /// </summary>
    /// <returns>Результаты перемещения (true - удалось переместиться,false - неудача)</returns>
    protected bool MoveRight() => MoveTo(MovementDirection.Right);

    /// <summary>
    /// Перемещение вниз
    /// </summary>
    /// <returns>Результаты перемещения (true - удалось переместиться,false - неудача)</returns>
    protected bool MoveDown() => MoveTo(MovementDirection.Down);

    /// <summary>
    /// Перемещение вверх
    /// </summary>
    /// <returns>Результаты перемещения (true - удалось переместиться,false - неудача)</returns>
    protected bool MoveUp() => MoveTo(MovementDirection.Up);

    /// <summary>
    /// 
    /// </summary>
    public void MakeStep()
    {

        if (_state != StrategyStatus.InProgress) return;

        if (IsTargetDestination())
        {

            _state = StrategyStatus.Finish;
            return;

        }

        MoveToTarget();

    }

    /// <summary>
    /// Параметры объекта
    /// </summary>
    protected ObjectParameters? GetObjectParameters => _moveableObject?.GetObjectPosition;

    /// <summary>
    /// Шаг объекта
    /// </summary>
    /// <returns></returns>
    protected int? GetStep()
    {
        if (_state != StrategyStatus.InProgress) return null;

        return _moveableObject?.GetStep;
    }

    /// <summary>
    /// Перемещение к цели
    /// </summary>
    /// <returns></returns>
    protected abstract void MoveToTarget();

    /// <summary>
    /// Достигнута ли цель
    /// </summary>
    /// <returns></returns>
    protected abstract bool IsTargetDestination();

    /// <summary>
    /// Попытка перемещения в требуемом направлении
    /// </summary>
    /// <param name="movementDirection"></param>
    /// <returns></returns>
    private bool MoveTo(MovementDirection movementDirection)
    {
        if (_state != StrategyStatus.InProgress) return false;

        return _moveableObject?.TryMoveableObject(movementDirection) ?? false;
    }
}
